/*

   Lectura remota de un directorio usando sockets pertenecientes
   a la familia TCP, en modo conexion.
   Codigo del servidor

   NOTA: Difiere del de las notas en el hecho de que este incluye
   una funcion de aborto en caso de teclear <ctrl><c>. Tambien
   la variable sin se cambio por sind

   Nombre Archivo: tcpserver.c
   Archivos relacionados: lee_dir.c tcpclient.c 
   Fecha: octubre 97

   Compilacion: cc tcpserver.c lee_dir.c -lnsl -lsocket -o tcpserver

   Ejecucion: tcpserver &

*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>

#define  DIRSIZE   2048      /* longitud maxima parametro entrada/salida */
#define  PUERTO   12345	     /* numero puerto arbitrario */

int                  sd, sd_actual;  /* descriptores de sockets */
int                  addrlen;        /* lomgitud direcciones */
struct sockaddr_in   sind, pin;      /* direcciones sockets cliente u servidor */


/*  procedimiento de aborte del servidor, si llega una senal SIGINT */
/* ( <ctrl> <c> ) se cierra el socket y se aborta el programa       */
   

void aborta()
{
   printf("....abortando el proceso servidor \n");
   close(sd);
   close(sd_actual);
   exit(1);
}


main()
{
	char  dir[DIRSIZE];	     /* parametro entrada y salida */

        signal(SIGINT, aborta);      /* activando la senal SIGINT */

/* obtencion de un socket tipo internet */
	if ( (sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

/* asignar direcciones en la estructura de direcciones */
	sind.sin_family = AF_INET;
	sind.sin_addr.s_addr = INADDR_ANY;   /* INADDR_ANY=0x000000 = yo mismo */
	sind.sin_port = htons(PUERTO);       /*  convirtiendo a formato red */

/* asociando del socket al numero de puerto */
	if ( bind(sd, (struct sockaddr *)&sind, sizeof(sind)) == -1) {
		perror("bind");
		exit(1);
	}

/* ponerse a escuchar a traves del socket */

	if (listen(sd, 5) == -1) {
		perror("listen");
		exit(1);
	}

/* esperando que un cliente solicite un servicio */

	if ( (sd_actual = accept(sd, (struct sockaddr *)&pin, &addrlen)) == -1) {
		perror("accept");
		exit(1);
	}
/* tomar un mensaje del cliente */

	if ( recv(sd_actual, dir, sizeof(dir), 0) == -1) {
		perror("recv");
		exit(1);
	}

/* leyendo el directorio */

	lee_dir(dir);

/* enviando la respuesta del servicio */

	if ( send(sd_actual, dir, strlen(dir), 0) == -1) {
		perror("send");
		exit(1);
	}

/* cerrar los dos sockets */
	close(sd_actual);  
        close(sd);
        printf("Conexion cerrada\n");

}
