/*

   Programa de lectura de un directorio remoto en base a 
   sockets de la familia internet con un protocolo orinetado no conexion
   Codigo del cliente.
 
   NOTA: Difiere del de las notas en el hecho de que este incluye
   una funcion de aborto en caso de teclear <ctrl><c>. Tambien
   en la parte de recepcion de la peticion se identifica al 
   cliente, por lo que se utiliza otras variables: dirlen, 
   nombre_clt y host_clt.

   Nombre Archivo: udpserver.c
   Archivos relacionados: lee_dir.c udpserver.c 
   Fecha: octubre 97

   Compilacion: cc udpserver.c lee_dir.c -lnsl -lsocket -o udpserver

   Ejecucion: udpserver <puerto> &

*/


#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <sys/errno.h>
#include <signal.h>

#define DIRSIZE    2048   /* longitud parametro entrada/salida */

int                  puerto;   /* numero de puerto */ 
int     	     sd;       /* descriptores de sockets */
int     	     addrlen;  /* lingitud direccion */ 
int                  cc;       /* regreso llamada recvfrom() */
struct sockaddr_in   myaddr;   /* mi direccion internet, servidor */
struct sockaddr_in   claddr;   /* direccion internet del cliente */	

/*  procedimiento de aborte del servidor */

void aborta()
{
   printf("....abortando el proceso servidor \n");
   close(sd);
   exit(1);
}


main(argc, argv)
   int   argc;
   char *argv[];
{
	char  		dir[DIRSIZE];	  /* parametro entrada y salida */
        char            *nombre_clt;      /* nombre del cliente */
        int             dirlen;           /* longitud direccion cliente */ 
        struct hostent  *host_clt;        /* estructura datos cliente */

/* activando la senal SIGINT, (ctrl-c) */

        signal(SIGINT, aborta);    

/* verificacion parametros */

        if (argc != 2) {
            fprintf(stderr,"Error, uso: %s <puerto> \n",argv[0]);
            exit(1);
        }
        puerto = atoi(argv[1]);

/* llenar la estructura de direcciones con la informacion del host */

	myaddr.sin_family = AF_INET;
	myaddr.sin_addr.s_addr = INADDR_ANY;                    
	myaddr.sin_port = puerto; 

/* obtencion de un socket tipo internet */

	if ( (sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

/* asociando del socket al numero de puerto */

	if (bind(sd, (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in)) == - 1) {
		perror("bind");
		exit(1);
	}

/* tomar el mensaje cuando este es enviado */

	addrlen = sizeof(struct sockaddr_in);
	cc = recvfrom(sd, dir, sizeof(dir), 0, (struct sockaddr *)&claddr, &addrlen);
	if ( cc == -1) {
		perror("recv");
		exit(1);
	}

/* identificando al emisor (no olvidar el & en el primer parametro de gethostbyaddr() ) */

        dirlen = sizeof(claddr.sin_addr.s_addr);     /* casi siempre tiene un valor de 4 */
        host_clt = gethostbyaddr((void *)&claddr.sin_addr.s_addr,dirlen , AF_INET);

        if (host_clt == NULL) 
          nombre_clt = "desconocido";  
        else 
          nombre_clt = host_clt->h_name; 

	printf("El mensaje [%s] fue recibido de: %s \n",dir, nombre_clt);
        printf("con direccion:  %d \n",(int)&claddr.sin_addr.s_addr);

/* realizando el servicio */

        lee_dir(dir);

/* enviar la respuesta al cliente */

	if (sendto(sd, dir, sizeof(dir), 0, (struct sockaddr *)&claddr, addrlen) == -1) {
		/* perror("sendto"); */
               printf("error \n");
		exit(1);
	}

/* conecctionless - no es necesario el accept() no hay socket extra que cerrar */

	close(sd);
}



